@@ -5,6 +5,9 @@ if [ -z $DESTDIR ]; then
 
     # Update icon cache
     gtk-update-icon-cache -f -t $PREFIX/share/icons/hicolor
+    
+    # Update desktop database cache
+    update-desktop-database
 
     # Install new schemas
     #glib-compile-schemas $PREFIX/share/glib-2.0/schemas/
