@@ -1,15 +1,18 @@
 # SPDX-License-Identifier: GPL-2.0-or-later
 
 from gettext import gettext as _
+from typing import Optional
 
 from .leddialog import LedDialog
 from .mousemap import MouseMap
 from .optionbutton import OptionButton
-from .ratbagd import RatbagdLed
+from .ratbagd import RatbagdDevice, RatbagdLed, RatbagdProfile
+from .util.gobject import connect_signal_with_weak_ref
 
 import gi
+
 gi.require_version("Gtk", "3.0")
-from gi.repository import Gtk  # noqa
+from gi.repository import Gtk, GObject  # noqa
 
 
 class LedsPage(Gtk.Box):
@@ -17,7 +20,9 @@ class LedsPage(Gtk.Box):
 
     __gtype_name__ = "LedsPage"
 
-    def __init__(self, ratbagd_device, *args, **kwargs):
+    def __init__(
+        self, ratbagd_device: RatbagdDevice, profile: RatbagdProfile, *args, **kwargs
+    ) -> None:
         """Instantiates a new LedsPage.
 
         @param ratbag_device The ratbag device to configure, as
@@ -25,48 +30,43 @@ class LedsPage(Gtk.Box):
         """
         Gtk.Box.__init__(self, *args, **kwargs)
         self._device = ratbagd_device
-        self._device.connect("active-profile-changed",
-                             self._on_active_profile_changed)
-        self._profile = None
+
+        self._profile = profile
 
         self._mousemap = MouseMap("#Leds", self._device, spacing=20, border_width=20)
         self.pack_start(self._mousemap, True, True, 0)
         self._sizegroup = Gtk.SizeGroup(mode=Gtk.SizeGroupMode.HORIZONTAL)
 
-        self._set_profile(self._device.active_profile)
-        self.show_all()
-
-    def _set_profile(self, profile):
-        self._profile = profile
         for led in profile.leds:
             mode = _(RatbagdLed.LED_DESCRIPTION[led.mode])
             button = OptionButton(mode)
             button.connect("clicked", self._on_button_clicked, led)
-            led.connect("notify::mode", self._on_led_mode_changed, button)
-            self._mousemap.add(button, "#led{}".format(led.index))
+
+            connect_signal_with_weak_ref(
+                self, led, "notify::mode", self._on_led_mode_changed, button
+            )
+
+            self._mousemap.add(button, f"#led{led.index}")
             self._sizegroup.add_widget(button)
 
-    def _on_active_profile_changed(self, device, profile):
-        # Disconnect the notify::mode signal on the old profile's LEDs.
-        for led in self._profile.leds:
-            led.disconnect_by_func(self._on_led_mode_changed)
-        # Clear the MouseMap of any children.
-        self._mousemap.foreach(Gtk.Widget.destroy)
-        # Repopulate the MouseMap.
-        self._set_profile(profile)
+        self.show_all()
 
-    def _on_led_mode_changed(self, led, pspec, button):
+    def _on_led_mode_changed(
+        self, led: RatbagdLed, pspec: Optional[GObject.ParamSpec], button: OptionButton
+    ) -> None:
         mode = _(RatbagdLed.LED_DESCRIPTION[led.mode])
         button.set_label(mode)
 
-    def _on_button_clicked(self, button, led):
+    def _on_button_clicked(self, button: OptionButton, led: RatbagdLed) -> None:
         # Presents the LedDialog to configure the LED corresponding to the
         # clicked button.
         dialog = LedDialog(led, transient_for=self.get_toplevel())
         dialog.connect("response", self._on_dialog_response, led)
         dialog.present()
 
-    def _on_dialog_response(self, dialog, response, led):
+    def _on_dialog_response(
+        self, dialog: LedDialog, response: Gtk.ResponseType, led: RatbagdLed
+    ) -> None:
         # The user either pressed cancel or apply. If it's apply, apply the
         # changes before closing the dialog, otherwise just close the dialog.
         if response == Gtk.ResponseType.APPLY:
