@@ -1,6 +1,4 @@
-# vim: set expandtab shiftwidth=4 tabstop=4:
-#
-# Copyright 2016 Red Hat, Inc.
+# Copyright 2016-2019 Red Hat, Inc.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -28,7 +26,8 @@ import hashlib
 from enum import IntEnum
 from evdev import ecodes
 from gettext import gettext as _
-from gi.repository import Gio, GLib, GObject  # noqa
+from gi.repository import Gio, GLib, GObject
+from typing import List, Optional, Tuple, Union
 
 
 # Deferred translations, see https://docs.python.org/3/library/gettext.html#deferred-translations
@@ -36,10 +35,18 @@ def N_(x):
     return x
 
 
+def evcode_to_str(evcode: int) -> str:
+    # Values in ecodes.keys are stored as either a str or list[str].
+    value = ecodes.keys[evcode]
+    if isinstance(value, list):
+        return value[0]
+    return value
+
+
 class RatbagErrorCode(IntEnum):
     SUCCESS = 0
 
-    """An error occured on the device. Either the device is not a libratbag
+    """An error occurred on the device. Either the device is not a libratbag
     device or communication with the device failed."""
     DEVICE = -1000
 
@@ -51,7 +58,7 @@ class RatbagErrorCode(IntEnum):
     outside of the legal or supported range."""
     VALUE = -1002
 
-    """A low-level system error has occured, e.g. a failure to access files
+    """A low-level system error has occurred, e.g. a failure to access files
     that should be there. This error is usually unrecoverable and libratbag will
     print a log message with details about the error."""
     SYSTEM = -1003
@@ -62,65 +69,74 @@ class RatbagErrorCode(IntEnum):
     IMPLEMENTATION = -1004
 
 
-class RatbagdIncompatible(Exception):
+class RatbagDeviceType(IntEnum):
+    """DeviceType property specified in the .device files"""
+
+    """There was no DeviceType specified for this device"""
+    UNSPECIFIED = 0
+
+    """Device is specified as anything other than a mouse or a keyboard"""
+    OTHER = 1
+
+    """Device is specified as a mouse"""
+    MOUSE = 2
+
+    """Device is specified as a keyboard"""
+    KEYBOARD = 3
+
+
+class RatbagdIncompatibleError(Exception):
     """ratbagd is incompatible with this client"""
+
     def __init__(self, ratbagd_version, required_version):
         super().__init__()
         self.ratbagd_version = ratbagd_version
         self.required_version = required_version
-        self.message = "ratbagd API version is {} but we require {}".format(ratbagd_version, required_version)
+        self.message = f"ratbagd API version is {ratbagd_version} but we require {required_version}"
 
     def __str__(self):
         return self.message
 
 
-class RatbagdUnavailable(Exception):
+class RatbagdUnavailableError(Exception):
     """Signals DBus is unavailable or the ratbagd daemon is not available."""
-    pass
 
 
-class RatbagdDBusTimeout(Exception):
+class RatbagdDBusTimeoutError(Exception):
     """Signals that a timeout occurred during a DBus method call."""
-    pass
 
 
 class RatbagError(Exception):
     """A common base exception to catch any ratbag exception."""
-    pass
 
 
-class RatbagErrorDevice(RatbagError):
+class RatbagDeviceError(RatbagError):
     """An exception corresponding to RatbagErrorCode.DEVICE."""
-    pass
 
 
-class RatbagErrorCapability(RatbagError):
+class RatbagCapabilityError(RatbagError):
     """An exception corresponding to RatbagErrorCode.CAPABILITY."""
-    pass
 
 
-class RatbagErrorValue(RatbagError):
+class RatbagValueError(RatbagError):
     """An exception corresponding to RatbagErrorCode.VALUE."""
-    pass
 
 
-class RatbagErrorSystem(RatbagError):
+class RatbagSystemError(RatbagError):
     """An exception corresponding to RatbagErrorCode.SYSTEM."""
-    pass
 
 
-class RatbagErrorImplementation(RatbagError):
+class RatbagImplementationError(RatbagError):
     """An exception corresponding to RatbagErrorCode.IMPLEMENTATION."""
-    pass
 
 
 """A table mapping RatbagErrorCode values to RatbagError* exceptions."""
 EXCEPTION_TABLE = {
-    RatbagErrorCode.DEVICE: RatbagErrorDevice,
-    RatbagErrorCode.CAPABILITY: RatbagErrorCapability,
-    RatbagErrorCode.VALUE: RatbagErrorValue,
-    RatbagErrorCode.SYSTEM: RatbagErrorSystem,
-    RatbagErrorCode.IMPLEMENTATION: RatbagErrorImplementation
+    RatbagErrorCode.DEVICE: RatbagDeviceError,
+    RatbagErrorCode.CAPABILITY: RatbagCapabilityError,
+    RatbagErrorCode.VALUE: RatbagValueError,
+    RatbagErrorCode.SYSTEM: RatbagSystemError,
+    RatbagErrorCode.IMPLEMENTATION: RatbagImplementationError,
 }
 
 
@@ -134,31 +150,33 @@ class _RatbagdDBus(GObject.GObject):
             try:
                 _RatbagdDBus._dbus = Gio.bus_get_sync(Gio.BusType.SYSTEM, None)
             except GLib.Error as e:
-                raise RatbagdUnavailable(e.message)
+                raise RatbagdUnavailableError(e.message) from e
 
         ratbag1 = "org.freedesktop.ratbag1"
-        if os.environ.get('RATBAG_TEST'):
+        if os.environ.get("RATBAG_TEST"):
             ratbag1 = "org.freedesktop.ratbag_devel1"
 
         if object_path is None:
-            object_path = "/" + ratbag1.replace('.', '/')
+            object_path = "/" + ratbag1.replace(".", "/")
 
         self._object_path = object_path
-        self._interface = "{}.{}".format(ratbag1, interface)
+        self._interface = f"{ratbag1}.{interface}"
 
         try:
-            self._proxy = Gio.DBusProxy.new_sync(_RatbagdDBus._dbus,
-                                                 Gio.DBusProxyFlags.NONE,
-                                                 None,
-                                                 ratbag1,
-                                                 object_path,
-                                                 self._interface,
-                                                 None)
+            self._proxy = Gio.DBusProxy.new_sync(
+                _RatbagdDBus._dbus,
+                Gio.DBusProxyFlags.NONE,
+                None,
+                ratbag1,
+                object_path,
+                self._interface,
+                None,
+            )
         except GLib.Error as e:
-            raise RatbagdUnavailable(e.message)
+            raise RatbagdUnavailableError(e.message) from e
 
         if self._proxy.get_name_owner() is None:
-            raise RatbagdUnavailable("No one currently owns {}".format(ratbag1))
+            raise RatbagdUnavailableError(f"No one currently owns {ratbag1}")
 
         self._proxy.connect("g-properties-changed", self._on_properties_changed)
         self._proxy.connect("g-signal", self._on_signal_received)
@@ -186,6 +204,12 @@ class _RatbagdDBus(GObject.GObject):
             return p.unpack()
         return p
 
+    def _get_dbus_property_nonnull(self, property: str):
+        p = self._get_dbus_property(property)
+        if p is None:
+            raise ValueError(f"D-Bus API returned `None` for property {property}")
+        return p
+
     def _set_dbus_property(self, property, type, value, readwrite=True):
         # Sets a cached property on the bus.
 
@@ -193,12 +217,16 @@ class _RatbagdDBus(GObject.GObject):
         # org.freedesktop.DBus.Properties.Set we need to wrap that again
         # into a (ssv), where v is our value's variant.
         # args to .Set are "interface name", "function name",  value-variant
-        val = GLib.Variant("{}".format(type), value)
+        val = GLib.Variant(f"{type}", value)
         if readwrite:
             pval = GLib.Variant("(ssv)", (self._interface, property, val))
-            self._proxy.call_sync("org.freedesktop.DBus.Properties.Set",
-                                  pval, Gio.DBusCallFlags.NO_AUTO_START,
-                                  2000, None)
+            self._proxy.call_sync(
+                "org.freedesktop.DBus.Properties.Set",
+                pval,
+                Gio.DBusCallFlags.NO_AUTO_START,
+                2000,
+                None,
+            )
 
         # This is our local copy, so we don't have to wait for the async
         # update
@@ -212,22 +240,21 @@ class _RatbagdDBus(GObject.GObject):
         # appropriate RatbagError* or RatbagdDBus* exception, or GLib.Error if
         # it is an unexpected exception that probably shouldn't be passed up to
         # the UI.
-        val = GLib.Variant("({})".format(type), value)
+        val = GLib.Variant(f"({type})", value)
         try:
-            res = self._proxy.call_sync(method, val,
-                                        Gio.DBusCallFlags.NO_AUTO_START,
-                                        2000, None)
+            res = self._proxy.call_sync(
+                method, val, Gio.DBusCallFlags.NO_AUTO_START, 2000, None
+            )
             if res in EXCEPTION_TABLE:
                 raise EXCEPTION_TABLE[res]
             return res.unpack()[0]  # Result is always a tuple
         except GLib.Error as e:
             if e.code == Gio.IOErrorEnum.TIMED_OUT:
-                raise RatbagdDBusTimeout(e.message)
-            else:
-                # Unrecognized error code; print the message to stderr and raise
-                # the GLib.Error.
-                print(e.message, file=sys.stderr)
-                raise
+                raise RatbagdDBusTimeoutError(e.message) from e
+
+            # Unrecognized error code.
+            print(e.message, file=sys.stderr)
+            raise
 
     def __eq__(self, other):
         return other and self._object_path == other._object_path
@@ -238,25 +265,28 @@ class Ratbagd(_RatbagdDBus):
     through ratbagd; actual interaction with the devices is via the
     RatbagdDevice, RatbagdProfile, RatbagdResolution and RatbagdButton objects.
 
-    Throws RatbagdUnavailable when the DBus service is not available.
+    Throws RatbagdUnavailableError when the DBus service is not available.
     """
 
     __gsignals__ = {
-        "device-added":
-            (GObject.SignalFlags.RUN_FIRST, None, (GObject.TYPE_PYOBJECT,)),
-        "device-removed":
-            (GObject.SignalFlags.RUN_FIRST, None, (GObject.TYPE_PYOBJECT,)),
-        "daemon-disappeared":
-            (GObject.SignalFlags.RUN_FIRST, None, ()),
+        "device-added": (GObject.SignalFlags.RUN_FIRST, None, (GObject.TYPE_PYOBJECT,)),
+        "device-removed": (
+            GObject.SignalFlags.RUN_FIRST,
+            None,
+            (GObject.TYPE_PYOBJECT,),
+        ),
+        "daemon-disappeared": (GObject.SignalFlags.RUN_FIRST, None, ()),
     }
 
     def __init__(self, api_version):
         super().__init__("Manager", None)
         result = self._get_dbus_property("Devices")
         if result is None and not self._proxy.get_cached_property_names():
-            raise RatbagdUnavailable("Make sure it is running and your user is in the required groups.")
+            raise RatbagdUnavailableError(
+                "Make sure it is running and your user is in the required groups."
+            )
         if self.api_version != api_version:
-            raise RatbagdIncompatible(self.api_version or -1, api_version)
+            raise RatbagdIncompatibleError(self.api_version or -1, api_version)
         self._devices = [RatbagdDevice(objpath) for objpath in result or []]
         self._proxy.connect("notify::g-name-owner", self._on_name_owner_changed)
 
@@ -264,15 +294,20 @@ class Ratbagd(_RatbagdDBus):
         self.emit("daemon-disappeared")
 
     def _on_properties_changed(self, proxy, changed_props, invalidated_props):
-        if "Devices" in changed_props.keys():
+        try:
+            new_device_object_paths = changed_props["Devices"]
+        except KeyError:
+            # Different property changed, skip.
+            pass
+        else:
             object_paths = [d._object_path for d in self._devices]
-            for object_path in changed_props["Devices"]:
+            for object_path in new_device_object_paths:
                 if object_path not in object_paths:
                     device = RatbagdDevice(object_path)
                     self._devices.append(device)
                     self.emit("device-added", device)
             for device in self.devices:
-                if device._object_path not in changed_props["Devices"]:
+                if device._object_path not in new_device_object_paths:
                     self._devices.remove(device)
                     self.emit("device-removed", device)
             self.notify("devices")
@@ -304,10 +339,12 @@ class RatbagdDevice(_RatbagdDBus):
     """Represents a ratbagd device."""
 
     __gsignals__ = {
-        "active-profile-changed":
-            (GObject.SignalFlags.RUN_FIRST, None, (GObject.TYPE_PYOBJECT,)),
-        "resync":
-            (GObject.SignalFlags.RUN_FIRST, None, ()),
+        "active-profile-changed": (
+            GObject.SignalFlags.RUN_FIRST,
+            None,
+            (GObject.TYPE_PYOBJECT,),
+        ),
+        "resync": (GObject.SignalFlags.RUN_FIRST, None, ()),
     }
 
     def __init__(self, object_path):
@@ -321,7 +358,7 @@ class RatbagdDevice(_RatbagdDBus):
             profile.connect("notify::is-active", self._on_active_profile_changed)
 
         # Use a SHA1 of our object path as our device's ID
-        self._id = hashlib.sha1(object_path.encode('utf-8')).hexdigest()
+        self._id = hashlib.sha1(object_path.encode("utf-8")).hexdigest()
 
     def _on_signal_received(self, proxy, sender_name, signal_name, parameters):
         if signal_name == "Resync":
@@ -349,6 +386,16 @@ class RatbagdDevice(_RatbagdDBus):
         """The device name, usually provided by the kernel."""
         return self._get_dbus_property("Name")
 
+    @GObject.Property
+    def device_type(self):
+        """The device type, see RatbagDeviceType"""
+        return RatbagDeviceType(self._get_dbus_property("DeviceType"))
+
+    @GObject.Property
+    def firmware_version(self):
+        """The firmware version of the device."""
+        return self._get_dbus_property("FirmwareVersion")
+
     @GObject.Property
     def profiles(self):
         """A list of RatbagdProfile objects provided by this device."""
@@ -359,12 +406,15 @@ class RatbagdDevice(_RatbagdDBus):
         """The currently active profile. This is a non-DBus property computed
         over the cached list of profiles. In the unlikely case that your device
         driver is misconfigured and there is no active profile, this returns
-        the first profile."""
+        `None`."""
         for profile in self._profiles:
             if profile.is_active:
                 return profile
-        print("No active profile. Please report this bug to the libratbag developers", file=sys.stderr)
-        return self._profiles[0]
+        print(
+            "No active profile. Please report this bug to the libratbag developers",
+            file=sys.stderr,
+        )
+        return None
 
     def commit(self):
         """Commits all changes made to the device.
@@ -375,10 +425,6 @@ class RatbagdDevice(_RatbagdDBus):
         device. No further interaction is required by the client.
         """
         self._dbus_call("Commit", "")
-        for profile in self._profiles:
-            if profile.dirty:
-                profile._dirty = False
-                profile.notify("dirty")
 
 
 class RatbagdProfile(_RatbagdDBus):
@@ -391,8 +437,12 @@ class RatbagdProfile(_RatbagdDBus):
 
     def __init__(self, object_path):
         super().__init__("Profile", object_path)
-        self._dirty = False
         self._active = self._get_dbus_property("IsActive")
+        self._angle_snapping = self._get_dbus_property("AngleSnapping")
+        self._debounce = self._get_dbus_property("Debounce")
+        self._dirty = self._get_dbus_property("IsDirty")
+        self._disabled = self._get_dbus_property("Disabled")
+        self._report_rate = self._get_dbus_property("ReportRate")
 
         # FIXME: if we start adding and removing objects from any of these
         # lists, things will break!
@@ -408,29 +458,82 @@ class RatbagdProfile(_RatbagdDBus):
         self._leds = [RatbagdLed(objpath) for objpath in result]
         self._subscribe_dirty(self._leds)
 
-    def _subscribe_dirty(self, objects):
+    def _subscribe_dirty(self, objects: List[GObject.GObject]):
         for obj in objects:
             obj.connect("notify", self._on_obj_notify)
 
-    def _on_obj_notify(self, obj, pspec):
+    def _on_obj_notify(self, obj: GObject.GObject, pspec: Optional[GObject.ParamSpec]):
         if not self._dirty:
             self._dirty = True
             self.notify("dirty")
 
     def _on_properties_changed(self, proxy, changed_props, invalidated_props):
-        if "IsActive" in changed_props.keys():
+        try:
+            angle_snapping = changed_props["AngleSnapping"]
+        except KeyError:
+            # Different property changed, skip.
+            pass
+        else:
+            if angle_snapping != self._angle_snapping:
+                self._angle_snapping = angle_snapping
+                self.notify("angle-snapping")
+
+        try:
+            debounce = changed_props["Debounce"]
+        except KeyError:
+            # Different property changed, skip.
+            pass
+        else:
+            if debounce != self._debounce:
+                self._debounce = debounce
+                self.notify("debounce")
+
+        try:
+            disabled = changed_props["Disabled"]
+        except KeyError:
+            # Different property changed, skip.
+            pass
+        else:
+            if disabled != self._disabled:
+                self._disabled = disabled
+                self.notify("disabled")
+
+        try:
             active = changed_props["IsActive"]
+        except KeyError:
+            # Different property changed, skip.
+            pass
+        else:
             if active != self._active:
                 self._active = active
                 self.notify("is-active")
-                self._on_obj_notify(None, None)
+
+        try:
+            dirty = changed_props["IsDirty"]
+        except KeyError:
+            # Different property changed, skip.
+            pass
+        else:
+            if dirty != self._dirty:
+                self._dirty = dirty
+                self.notify("dirty")
+
+        try:
+            report_rate = changed_props["ReportRate"]
+        except KeyError:
+            # Different property changed, skip.
+            pass
+        else:
+            if report_rate != self._report_rate:
+                self._report_rate = report_rate
+                self.notify("report-rate")
 
     @GObject.Property
     def capabilities(self):
         """The capabilities of this profile as an array. Capabilities not
         present on the profile are not in the list. Thus use e.g.
 
-        if RatbagdProfile.CAP_WRITABLE_NAME is in profile.capabilities:
+        if RatbagdProfile.CAP_WRITABLE_NAME in profile.capabilities:
             do something
         """
         return self._get_dbus_property("Capabilities") or []
@@ -458,21 +561,21 @@ class RatbagdProfile(_RatbagdDBus):
         return self._dirty
 
     @GObject.Property
-    def enabled(self):
-        """tells if the profile is enabled."""
-        return self._get_dbus_property("Enabled")
+    def disabled(self):
+        """tells if the profile is disabled."""
+        return self._disabled
 
-    @enabled.setter
-    def enabled(self, enabled):
+    @disabled.setter
+    def disabled(self, disabled):
         """Enable/Disable this profile.
 
-        @param enabled The new state, as boolean"""
-        self._set_dbus_property("Enabled", "b", enabled)
+        @param disabled The new state, as boolean"""
+        self._set_dbus_property("Disabled", "b", disabled)
 
     @GObject.Property
-    def report_rate(self):
+    def report_rate(self) -> int:
         """The report rate in Hz."""
-        return self._get_dbus_property("ReportRate")
+        return self._report_rate
 
     @report_rate.setter
     def report_rate(self, rate):
@@ -482,15 +585,42 @@ class RatbagdProfile(_RatbagdDBus):
         """
         self._set_dbus_property("ReportRate", "u", rate)
 
-        if not self._dirty:
-            self._dirty = True
-            self.notify("dirty")
-
     @GObject.Property
     def report_rates(self):
         """The list of supported report rates"""
         return self._get_dbus_property("ReportRates") or []
 
+    @GObject.Property
+    def angle_snapping(self):
+        """The angle snapping option."""
+        return self._angle_snapping
+
+    @angle_snapping.setter
+    def angle_snapping(self, value):
+        """Set the angle snapping option.
+
+        @param value The angle snapping option as int
+        """
+        self._set_dbus_property("AngleSnapping", "i", value)
+
+    @GObject.Property
+    def debounce(self):
+        """The button debounce time in ms."""
+        return self._debounce
+
+    @debounce.setter
+    def debounce(self, value):
+        """Set the button debounce time in ms.
+
+        @param value The button debounce time, as int
+        """
+        self._set_dbus_property("Debounce", "i", value)
+
+    @GObject.Property
+    def debounces(self):
+        """The list of supported debounce times"""
+        return self._get_dbus_property("Debounces") or []
+
     @GObject.Property
     def resolutions(self):
         """A list of RatbagdResolution objects with this profile's resolutions.
@@ -503,12 +633,15 @@ class RatbagdProfile(_RatbagdDBus):
         """The currently active resolution of this profile. This is a non-DBus
         property computed over the cached list of resolutions. In the unlikely
         case that your device driver is misconfigured and there is no active
-        resolution, this returns the first resolution."""
+        resolution, this returns `None`."""
         for resolution in self._resolutions:
             if resolution.is_active:
                 return resolution
-        print("No active resolution. Please report this bug to the libratbag developers", file=sys.stderr)
-        return self._resolutions[0]
+        print(
+            "No active resolution. Please report this bug to the libratbag developers",
+            file=sys.stderr,
+        )
+        return None
 
     @GObject.Property
     def buttons(self):
@@ -539,37 +672,91 @@ class RatbagdProfile(_RatbagdDBus):
 class RatbagdResolution(_RatbagdDBus):
     """Represents a ratbagd resolution."""
 
+    CAP_SEPARATE_XY_RESOLUTION = 1
+    CAP_DISABLE = 2
+
     def __init__(self, object_path):
         super().__init__("Resolution", object_path)
         self._active = self._get_dbus_property("IsActive")
         self._default = self._get_dbus_property("IsDefault")
+        self._disabled = self._get_dbus_property("IsDisabled")
+        self._resolution = self._convert_resolution_from_dbus(
+            self._get_dbus_property_nonnull("Resolution")
+        )
 
     def _on_properties_changed(self, proxy, changed_props, invalidated_props):
-        if "IsActive" in changed_props.keys():
+        try:
+            resolution = self._convert_resolution_from_dbus(changed_props["Resolution"])
+        except KeyError:
+            # Different property changed, skip.
+            pass
+        else:
+            if resolution != self._resolution:
+                self._resolution = resolution
+                self.notify("resolution")
+
+        try:
             active = changed_props["IsActive"]
+        except KeyError:
+            # Different property changed, skip.
+            pass
+        else:
             if active != self._active:
                 self._active = active
                 self.notify("is-active")
-        elif "IsDefault" in changed_props.keys():
+
+        try:
             default = changed_props["IsDefault"]
+        except KeyError:
+            # Different property changed, skip.
+            pass
+        else:
             if default != self._default:
                 self._default = default
                 self.notify("is-default")
 
+        try:
+            disabled = changed_props["IsDisabled"]
+        except KeyError:
+            # Different property changed, skip.
+            pass
+        else:
+            if disabled != self._disabled:
+                self._disabled = disabled
+                self.notify("is-disabled")
+
+    @GObject.Property
+    def capabilities(self):
+        """The capabilities of this resolution as an array. Capabilities not
+        present on the resolution are not in the list. Thus use e.g.
+
+        if resolution.CAP_DISABLE in resolution.capabilities:
+            do something
+        """
+        return self._get_dbus_property("Capabilities") or []
+
     @GObject.Property
     def index(self):
         """The index of this resolution."""
         return self._get_dbus_property("Index")
 
+    @staticmethod
+    def _convert_resolution_from_dbus(
+        res: Union[int, Tuple[int, int]]
+    ) -> Union[Tuple[int], Tuple[int, int]]:
+        """
+        Convert resolution from what D-Bus API retuns - either an int or a tuple of two ints, to a tuple of either one or two ints.
+        """
+        if isinstance(res, int):
+            return (res,)
+        return res
+
     @GObject.Property
     def resolution(self):
         """The resolution in DPI, either as single value tuple ``(res, )``
         or as tuple ``(xres, yres)``.
         """
-        res = self._get_dbus_property("Resolution")
-        if isinstance(res, int):
-            res = tuple([res])
-        return res
+        return self._resolution
 
     @resolution.setter
     def resolution(self, resolution):
@@ -579,11 +766,11 @@ class RatbagdResolution(_RatbagdDBus):
         """
         res = self.resolution
         if len(res) != len(resolution) or len(res) > 2:
-            raise ValueError('invalid resolution precision')
+            raise ValueError("invalid resolution precision")
         if len(res) == 1:
-            variant = GLib.Variant('u', resolution[0])
+            variant = GLib.Variant("u", resolution[0])
         else:
-            variant = GLib.Variant('(uu)', resolution)
+            variant = GLib.Variant("(uu)", resolution)
         self._set_dbus_property("Resolution", "v", variant)
 
     @GObject.Property
@@ -603,11 +790,10 @@ class RatbagdResolution(_RatbagdDBus):
         otherwise"""
         return self._default
 
-    def set_default(self):
-        """Set this resolution to be the default."""
-        ret = self._dbus_call("SetDefault", "")
-        self._set_dbus_property("IsDefault", "b", True, readwrite=False)
-        return ret
+    @GObject.Property
+    def is_disabled(self):
+        """True if this is currently disabled, False otherwise"""
+        return self._disabled
 
     def set_active(self):
         """Set this resolution to be the active one."""
@@ -615,6 +801,16 @@ class RatbagdResolution(_RatbagdDBus):
         self._set_dbus_property("IsActive", "b", True, readwrite=False)
         return ret
 
+    def set_default(self):
+        """Set this resolution to be the default."""
+        ret = self._dbus_call("SetDefault", "")
+        self._set_dbus_property("IsDefault", "b", True, readwrite=False)
+        return ret
+
+    def set_disabled(self, disable):
+        """Set this resolution to be disabled."""
+        return self._set_dbus_property("IsDisabled", "b", disable)
+
 
 class RatbagdButton(_RatbagdDBus):
     """Represents a ratbagd button."""
@@ -623,11 +819,12 @@ class RatbagdButton(_RatbagdDBus):
         NONE = 0
         BUTTON = 1
         SPECIAL = 2
+        KEY = 3
         MACRO = 4
 
     class ActionSpecial(IntEnum):
         INVALID = -1
-        UNKNOWN = (1 << 30)
+        UNKNOWN = 1 << 30
         DOUBLECLICK = (1 << 30) + 1
         WHEEL_LEFT = (1 << 30) + 2
         WHEEL_RIGHT = (1 << 30) + 3
@@ -718,8 +915,9 @@ class RatbagdButton(_RatbagdDBus):
         @param button The button to map to, as int
         """
         button = GLib.Variant("u", button)
-        self._set_dbus_property("Mapping", "(uv)",
-                                (RatbagdButton.ActionType.BUTTON, button))
+        self._set_dbus_property(
+            "Mapping", "(uv)", (RatbagdButton.ActionType.BUTTON, button)
+        )
 
     @GObject.Property
     def macro(self):
@@ -739,8 +937,9 @@ class RatbagdButton(_RatbagdDBus):
                      the button, as RatbagdMacro.
         """
         macro = GLib.Variant("a(uu)", macro.keys)
-        self._set_dbus_property("Mapping", "(uv)",
-                                (RatbagdButton.ActionType.MACRO, macro))
+        self._set_dbus_property(
+            "Mapping", "(uv)", (RatbagdButton.ActionType.MACRO, macro)
+        )
 
     @GObject.Property
     def special(self):
@@ -758,8 +957,21 @@ class RatbagdButton(_RatbagdDBus):
         @param special The special entry, as one of RatbagdButton.ActionSpecial
         """
         special = GLib.Variant("u", special)
-        self._set_dbus_property("Mapping", "(uv)",
-                                (RatbagdButton.ActionType.SPECIAL, special))
+        self._set_dbus_property(
+            "Mapping", "(uv)", (RatbagdButton.ActionType.SPECIAL, special)
+        )
+
+    @GObject.Property
+    def key(self):
+        type, key = self._mapping()
+        if type != RatbagdButton.ActionType.KEY:
+            return None
+        return key
+
+    @key.setter
+    def key(self, key):
+        key = GLib.Variant("u", key)
+        self._set_dbus_property("Mapping", "(uv)", (RatbagdButton.ActionType.KEY, key))
 
     @GObject.Property
     def action_type(self):
@@ -776,9 +988,17 @@ class RatbagdButton(_RatbagdDBus):
         """An array of possible values for ActionType."""
         return self._get_dbus_property("ActionTypes")
 
+    @GObject.Property
+    def disabled(self):
+        type, unused = self._mapping()
+        return type == RatbagdButton.ActionType.NONE
+
     def disable(self):
         """Disables this button."""
-        return self._dbus_call("Disable", "")
+        zero = GLib.Variant("u", 0)
+        self._set_dbus_property(
+            "Mapping", "(uv)", (RatbagdButton.ActionType.NONE, zero)
+        )
 
 
 class RatbagdMacro(GObject.Object):
@@ -786,25 +1006,18 @@ class RatbagdMacro(GObject.Object):
     linux/input.h and not those used by X.Org or any other higher layer such as
     Gdk."""
 
-    # All keys from ecodes.KEY have a KEY_ prefix. We strip it.
-    _PREFIX_LEN = len("KEY_")
-
     # Both a key press and release.
     _MACRO_KEY = 1000
 
     _MACRO_DESCRIPTION = {
-        RatbagdButton.Macro.KEY_PRESS: lambda key:
-            "↓{}".format(ecodes.KEY[key][RatbagdMacro._PREFIX_LEN:]),
-        RatbagdButton.Macro.KEY_RELEASE: lambda key:
-            "↑{}".format(ecodes.KEY[key][RatbagdMacro._PREFIX_LEN:]),
-        RatbagdButton.Macro.WAIT: lambda val:
-            "{}ms".format(val),
-        _MACRO_KEY: lambda key:
-            "↕{}".format(ecodes.KEY[key][RatbagdMacro._PREFIX_LEN:]),
+        RatbagdButton.Macro.KEY_PRESS: lambda key: f"↓{evcode_to_str(key)}",
+        RatbagdButton.Macro.KEY_RELEASE: lambda key: f"↑{evcode_to_str(key)}",
+        RatbagdButton.Macro.WAIT: lambda val: f"{val}ms",
+        _MACRO_KEY: lambda key: f"↕{evcode_to_str(key)}",
     }
 
     __gsignals__ = {
-        'macro-set': (GObject.SignalFlags.RUN_FIRST, None, ()),
+        "macro-set": (GObject.SignalFlags.RUN_FIRST, None, ()),
     }
 
     def __init__(self, **kwargs):
@@ -851,7 +1064,7 @@ class RatbagdMacro(GObject.Object):
 
         # Do not emit notify::keys for every key that we add.
         with ratbagd_macro.freeze_notify():
-            for (type, value) in macro:
+            for type, value in macro:
                 ratbagd_macro.append(type, value)
         return ratbagd_macro
 
@@ -877,12 +1090,6 @@ class RatbagdMacro(GObject.Object):
 class RatbagdLed(_RatbagdDBus):
     """Represents a ratbagd led."""
 
-    TYPE_LOGO = 1
-    TYPE_SIDE = 2
-    TYPE_BATTERY = 3
-    TYPE_DPI = 4
-    TYPE_WHEEL = 5
-
     class Mode(IntEnum):
         OFF = 0
         ON = 1
@@ -909,6 +1116,11 @@ class RatbagdLed(_RatbagdDBus):
     def __init__(self, object_path):
         super().__init__("Led", object_path)
 
+        self._brightness = self._get_dbus_property("Brightness")
+        self._color = self._get_dbus_property("Color")
+        self._effect_duration = self._get_dbus_property("EffectDuration")
+        self._mode: RatbagdLed.Mode = self._get_dbus_property_nonnull("Mode")
+
     @GObject.Property
     def index(self):
         """The index of this led."""
@@ -918,7 +1130,7 @@ class RatbagdLed(_RatbagdDBus):
     def mode(self):
         """This led's mode, one of Mode.OFF, Mode.ON, Mode.CYCLE and
         Mode.BREATHING."""
-        return self._get_dbus_property("Mode")
+        return self._mode
 
     @mode.setter
     def mode(self, mode):
@@ -937,7 +1149,7 @@ class RatbagdLed(_RatbagdDBus):
     @GObject.Property
     def color(self):
         """An integer triple of the current LED color."""
-        return self._get_dbus_property("Color")
+        return self._color
 
     @color.setter
     def color(self, color):
@@ -956,7 +1168,7 @@ class RatbagdLed(_RatbagdDBus):
     @GObject.Property
     def effect_duration(self):
         """The LED's effect duration in ms, values range from 0 to 10000."""
-        return self._get_dbus_property("EffectDuration")
+        return self._effect_duration
 
     @effect_duration.setter
     def effect_duration(self, effect_duration):
@@ -969,7 +1181,7 @@ class RatbagdLed(_RatbagdDBus):
     @GObject.Property
     def brightness(self):
         """The LED's brightness, values range from 0 to 255."""
-        return self._get_dbus_property("Brightness")
+        return self._brightness
 
     @brightness.setter
     def brightness(self, brightness):
@@ -978,3 +1190,44 @@ class RatbagdLed(_RatbagdDBus):
         @param brightness The new brightness, as int
         """
         self._set_dbus_property("Brightness", "u", brightness)
+
+    def _on_properties_changed(self, proxy, changed_props, invalidated_props):
+        try:
+            brightness = changed_props["Brightness"]
+        except KeyError:
+            # Different property changed, skip.
+            pass
+        else:
+            if brightness != self._brightness:
+                self._brightness = brightness
+                self.notify("brightness")
+
+        try:
+            color = changed_props["Color"]
+        except KeyError:
+            # Different property changed, skip.
+            pass
+        else:
+            if color != self._color:
+                self._color = color
+                self.notify("color")
+
+        try:
+            effect_duration = changed_props["EffectDuration"]
+        except KeyError:
+            # Different property changed, skip.
+            pass
+        else:
+            if effect_duration != self._effect_duration:
+                self._effect_duration = effect_duration
+                self.notify("effect-duration")
+
+        try:
+            mode = changed_props["Mode"]
+        except KeyError:
+            # Different property changed, skip.
+            pass
+        else:
+            if mode != self._mode:
+                self._mode = mode
+                self.notify("mode")
